﻿using PDF_Office.Model.PDFTool;
using PDF_Office.Properties;
using PDFSettings;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace PDF_Office.Helper
{
    public enum PDFToolType
    {
        Split,
        Extract,
        Insert,
        Compress,
        Merge,
        Print,
        Security,
        ConvertPDF,
        PDFToWord,
        PDFToExcel,
        PDFToPPT,
        ImageToPDF,
        OCR,
        WaterMark,
        HeaderFooter,
        BatesNumbers,
        Batch,
        Background,
        CompareDoc

    }
    public class PDFToolsHelper
    {
        public List<ToolItem> AllTools = null;
        public List<ToolItem> QuickTools = null;
        public PDFToolsHelper()
        {
            InitPDFTools();
        }

        private void InitPDFTools()
        {
            AllTools = new List<ToolItem>();
            InitAllTools();
            InitQuickTools();
        }

        private void InitAllTools()
        {
            string path = @"pack://application:,,,/Resources/HomeIcon/PDFTools/split.png";
            AddToolItem(PDFToolType.Split, path, "拆分", "Batch convert, compress, secure, watermark PDFs.");

            path = @"pack://application:,,,/Resources/HomeIcon/PDFTools/extract.png";
            AddToolItem(PDFToolType.Extract, path, "提取", "Batch convert, compress, secure, watermark PDFs.");

            path = @"pack://application:,,,/Resources/HomeIcon/PDFTools/insert.png";
            AddToolItem(PDFToolType.Insert, path, "插入", "Batch convert, compress, secure, watermark PDFs.");

            path = @"pack://application:,,,/Resources/HomeIcon/PDFTools/compress.png";
            AddToolItem(PDFToolType.Compress, path, "压缩", "Batch convert, compress, secure, watermark PDFs.");

            path = @"pack://application:,,,/Resources/HomeIcon/PDFTools/merge.png";
            AddToolItem(PDFToolType.Merge, path, "合并", "Batch convert, compress, secure, watermark PDFs.");

            path = @"pack://application:,,,/Resources/HomeIcon/PDFTools/print.png";
            AddToolItem(PDFToolType.Print, path, "打印", "Batch convert, compress, secure, watermark PDFs.");

            path = @"pack://application:,,,/Resources/HomeIcon/PDFTools/security.png";
            AddToolItem(PDFToolType.Security, path, "安全", "Batch convert, compress, secure, watermark PDFs.");

            path = @"pack://application:,,,/Resources/HomeIcon/PDFTools/convert.png";
            AddToolItem(PDFToolType.ConvertPDF, path, "转档PDF", "Batch convert, compress, secure, watermark PDFs.");

            path = @"pack://application:,,,/Resources/HomeIcon/PDFTools/pdf-to-word.png";
            AddToolItem(PDFToolType.PDFToWord, path, "PDF转Word", "Batch convert, compress, secure, watermark PDFs.");

            path = @"pack://application:,,,/Resources/HomeIcon/PDFTools/pdf-to-excel.png";
            AddToolItem(PDFToolType.PDFToExcel, path, "PDF转Excel", "Batch convert, compress, secure, watermark PDFs.");

            path = @"pack://application:,,,/Resources/HomeIcon/PDFTools/pdf-to-ppt.png";
            AddToolItem(PDFToolType.PDFToPPT, path, "PDF转PPT", "Batch convert, compress, secure, watermark PDFs.");

            path = @"pack://application:,,,/Resources/HomeIcon/PDFTools/image.png";
            AddToolItem(PDFToolType.ImageToPDF, path, "图片转PDF", "Batch convert, compress, secure, watermark PDFs.");

            path = @"pack://application:,,,/Resources/HomeIcon/PDFTools/ocr.png";
            AddToolItem(PDFToolType.OCR, path, "OCR", "Batch convert, compress, secure, watermark PDFs.");

            path = @"pack://application:,,,/Resources/HomeIcon/PDFTools/watermark.png";
            AddToolItem(PDFToolType.WaterMark, path, "水印", "Batch convert, compress, secure, watermark PDFs.");

            path = @"pack://application:,,,/Resources/HomeIcon/PDFTools/header&footer.png";
            AddToolItem(PDFToolType.HeaderFooter, path, "页眉页脚", "Batch convert, compress, secure, watermark PDFs.");

            path = @"pack://application:,,,/Resources/HomeIcon/PDFTools/batesnumbers.png";
            AddToolItem(PDFToolType.BatesNumbers, path, "贝茨Bates码", "Batch convert, compress, secure, watermark PDFs.");

            path = @"pack://application:,,,/Resources/HomeIcon/PDFTools/batch.png";
            AddToolItem(PDFToolType.Batch, path, "批量处理", "Batch convert, compress, secure, watermark PDFs.");

            path = @"pack://application:,,,/Resources/HomeIcon/PDFTools/background.png";
            AddToolItem(PDFToolType.Background, path, "背景", "Batch convert, compress, secure, watermark PDFs.");

            path = @"pack://application:,,,/Resources/HomeIcon/PDFTools/compress.png";
            AddToolItem(PDFToolType.CompareDoc, path, "文件对比", "Batch convert, compress, secure, watermark PDFs.");

           
        }

        private void ContianNewTools()
        {
            if (Settings.Default.AllPDFToolsList != null)
            {
                foreach(var item in AllTools)
                {
                    Settings.Default.AllPDFToolsList.FirstOrDefault(tool => tool.Tag == item.Tag);
                }
            }
        }

        private void AddToolItem(PDFToolType toolType, string imgPath,string title,string titleInfo,int id = 0)
        {
            ToolItem toolItem = new ToolItem();

            toolItem.Id = id;
            toolItem.Tag = (int)toolType;
            toolItem.TagStr = Enum.GetName(typeof(PDFToolType), toolType);
            toolItem.Image = imgPath;
            toolItem.Title = title;
            toolItem.TitleInfo = titleInfo;
            AllTools.Add(toolItem);
        }

        private void InitQuickTools()
        {
            QuickTools = AllTools.Take(8).ToList<ToolItem>();
        }
    }


}
