﻿using ComPDFKitViewer;
using PDF_Office.ViewModels;
using PDF_Office.Views.PropertyPanel.ViewModular;
using System;
using System.Threading;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Threading;

namespace PDF_Office.Views
{
    /// <summary>
    /// Interaction logic for ViewContent
    /// </summary>
    public partial class ViewContent : UserControl
    {
        private ViewContentViewModel viewModel = null;

        public ViewContent()
        {
            InitializeComponent();
            viewModel = (ViewContentViewModel)this.DataContext;
        }

        /// <summary>
        /// 阅读模式
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void RbtnReadMode_Click(object sender, RoutedEventArgs e)
        {
            if (viewModel != null)
            {
                App.IsBookMode = true;
                SetReadMode(true);
            }
        }

        /// <summary>
        /// 设置阅读模式下的控件
        /// </summary>
        /// <param name="isBookMode"></param>
        private void SetReadMode(bool isBookMode)
        {
            if (isBookMode)
            {
                viewModel.IsReadMode = Visibility.Collapsed;
                viewModel.IsPropertyOpen = false;
                //viewModel.ToolContentVisible = Visibility.Collapsed;
                //viewModel.ToolsBarContentVisible = Visibility.Collapsed;
                //viewModel.ConverterBarContentVisible = Visibility.Collapsed;
                this.ReadModeContent.Visibility = Visibility.Visible;
                this.RectangleReadMode.Visibility = Visibility.Collapsed;
                Grid.SetColumn(this.PDFViewerContent, 0);
                Grid.SetColumnSpan(this.PDFViewerContent, 4);
                Grid.SetRow(this.DocumentView, 0);
                Grid.SetRowSpan(this.DocumentView, 4);
            }
            else
            {
                viewModel.IsReadMode = Visibility.Visible;
                viewModel.IsPropertyOpen = true;
                //if (viewModel.CurrentBar == "TabItemConvert")
                //{
                //    viewModel.ConverterBarContentVisible = Visibility.Visible;
                //}
                //else if (viewModel.CurrentBar == "TabItemPageEdit")
                //{
                //    viewModel.ToolContentVisible = Visibility.Visible;
                //}
                //else
                //{
                //    viewModel.ToolsBarContentVisible = Visibility.Visible;
                //}
                //viewModel.ToolContentVisible = Visibility.Visible;

                this.ReadModeContent.Visibility = Visibility.Collapsed;
                this.RectangleReadMode.Visibility = Visibility.Visible;
                Grid.SetColumn(this.PDFViewerContent, 2);
                Grid.SetColumnSpan(this.PDFViewerContent, 1);
                Grid.SetRow(this.DocumentView, 2);
                Grid.SetRowSpan(this.DocumentView, 1);
            }
        }

        /// <summary>
        /// 退出阅读模式
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void UserControl_KeyDown(object sender, System.Windows.Input.KeyEventArgs e)
        {
            if (Key.Escape == e.Key)
            {
                if (viewModel != null)
                {
                    App.IsBookMode = false;
                    SetReadMode(false);
                }
            }
        }

        /// <summary>
        /// 鼠标移开页面控件，三秒后隐藏该控件
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private async void ReadModeContent_MouseLeave(object sender, RoutedEventArgs e)
        {
            await System.Threading.Tasks.Task.Delay(3000);
            //Thread.Sleep(3000);
            this.ReadModeContent.Visibility = Visibility.Collapsed;
            this.RectangleReadMode.Visibility = Visibility.Visible;
        }

        /// <summary>
        /// 鼠标移动到控件范围时显示控件
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void RectangleReadMode_MouseEnter(object sender, MouseEventArgs e)
        {
            Dispatcher.BeginInvoke(DispatcherPriority.Send, new Action(delegate
            {
                if (App.IsBookMode)
                {
                    this.ReadModeContent.Visibility = Visibility.Visible;
                    this.RectangleReadMode.Visibility = Visibility.Collapsed;
                }
            }));
        }
    }
}